$TITLE Stylized Numerical CGE Model
$OFFSYMLIST OFFSYMXREF

*## UNION MODEL WITH HIGH WEIGHT ON WAGES IN UTILITY FUNCTION
*## Thierfelder and Shiells, CH 14: "Trade and Labor Market Behavior" 
*## Contact thier@nadn.navy.mil with questions about GAMS code

*###################### SET DECLARATION #############################
 SET 
     f         factors /lab, cap/
     i         sectors / ag, mfg/
     ref(f)    reference input for f.o.c. ratio  /cap/
     nref(f)   non-reference inputs for f.o.c. ratio 
     ew(i,f)   efficiency wage sectors /mfg.lab/
     new(i,f)  non-efficiency wage sectors
     un(i,f)   unionized sectors /mfg.lab/
     nun(i,f)  non-unionized sectors

;

* Assign Sub-sets
 nref(f)          =  YES ;
 nref(f)$ref(f)   =  NO ;

* Assign Sectors with Efficiency Wage Payments to Labor Markets
 new(i,f)         =  YES ;
 new(i,f)$ew(i,f) =  NO ;

* To Turn off Efficiency Wage Behavior
 ew(i,f)          =  NO ;
 new(i,f)         =  YES ;

 DISPLAY EW, NEW ;

* Assign Sectors with Union Behavior
 nun(i,f)         =  YES ;
 nun(i,f)$un(i,f) =  NO ;

* To Turn off Union Behavior
* un(i,f)          =  NO ;
* nun(i,f)         =  YES ;

 display un, nun ;

 Alias (f,ff) ;
 Alias (i,j) ;

Parameter
    SIGMA(i)    CES trade substitution elasticity 
                  /ag   1.5
                   mfg  1.5 /

    OMEGA(i)    CET export transformtion elasticity
                  /ag   2.0
                   mfg  2.0 /
    E0(i)       Initializing exports
                 /ag   10
                  mfg  10  /
    M0(i)       Initializing imports
                 /ag   15
                  mfg  25 /
    X0(i)       Initializing value of production
                  /ag  100
                   mfg 100 / 
    Q0(i)       Initializing consumption possibility 
    D0(i)       initializing domestic component of demand
    PX0(i)      Initializing domestic production price
    PQ0(i)      Initializing composite consumption good price
    PWE0(i)     Initializing export price
    PWM0(i)     Initializing import price
    ER0         Exchange rate
    PDD0(i)     Initializing domestic price
    PED0(i)     Price of Exported Good including subsidy
    PMD0(i)     Price of Imported Good including tariff
    PDT0(i)     Price of Domestic Good including tax
    TM0(i)      Import Tariff 
    TE0(i)      Export Subsidy
    TD0(i)      Domestic Good Tax
    PINDEX0     Price Index 
    PWTS(i)     Weights for Price Index    

    RHOC(i)     CES import aggregation parameter
    RHOT(i)     CET export transformation parameter    
    ALPHA(i)    CET share parameter
    BETA(i)     CES share parameter
    A2(i)       CET constant
    A1(i)       CES constant

    FSAV0       Initializing Bilateral Current Account Bal
    GR0         Initializing Government Revenue
    Y0          Initializing National Income

* factor markets
   FDSC0(i,f)   initializing factor demand
                 /ag.lab  30
                  ag.cap  20
                  mfg.lab 20 
                  mfg.cap 40  /

   FCTRY0(i,f)   initializing factor income
* note, these numbers just create shares to allocate revenue from production
                 /ag.lab   10
                  ag.cap   15
                  mfg.lab  50
                  mfg.cap  20 /

   FS0(f)       factor supply
   WF0(f)       initializing factor price
   WFDIST0(i,f) factor price sectoral proportionality constants 
   YFCTR0(f)    factor income
   YFSECT0(i)   total factor income
 
   RHOP(i)       CES production function parameter
                  / ag  0.5
                    mfg 0.5 /

   DELTA(i,f)    CES production share parameter
   A3(i)         CES production constant
   SIGMAP(i)     Elasticity of Substitution in Production

* efficiency wage behavior
D2(i,f)             Probability of Being Falsely Accused of Shirking
D1(i,f)             Probability a Shirker Will Be Caught
QR(i,f)             Quit Rate
UTS(i,f)            Utility from Shirking
IR                  Interest Rate
MC0(i,f)            Initial Monitoring Cost Apparent as a Wage Markup
MONITOR0(i,f)       Total Monitoring Costs 

* union behavior
 MU(i,f)        Weight union puts on wage mark-up in its utility function
                /mfg.lab 0.9 /

 LBAR(i,f)      Reservation Employment Level for Union
 WSUB(i,f)      Wage Subsidy

* welfare
 CLIV0          Cost of Living Index
 UTIL0          Base Utility
 CLES(i)        Expenditure Shares
 ; 

*## INITIALIZE PRICES
  ER0                    = 1.0 ;
  tm0(i)                 = 0.0 ;
  tm0("mfg")             = 0.3 ;
  te0(i)                 = 0.0 ;
  td0(i)                 = 0.0 ;
  pdd0(i)                = 1.0 ;
  pdt0(i)                = (1+TD0(i))*PDD0(i) ;
  px0(i)                 = 1.0 ;
  PQ0(i)                 = 1.0 ; 
  ped0(i)                = 1.0 ;
  pmd0(i)                = 1.0 ;
  pwe0(i)                = ped0(i)/((1 + te0(i))*ER0) ;
  pwm0(i)                = pmd0(i)/((1 + tm0(i))*ER0) ;
  rhoc(i)                = (1/SIGMA(i)) - 1;
  rhot(i)                = (1/(-OMEGA(i))) - 1;

DISPLAY PWE0, PWM0, PMD0, PED0, PQ0, RHOC, RHOT ;

*## INITIALIZE QUANTITIES

  D0(i)      = (X0(i)*PX0(i) -  E0(i)*PED0(i) ) /PDD0(i) ;
  Q0(i)      = (PDT0(i)*D0(i) + M0(i)*PMD0(i) ) /PQ0(i) ;
  FSAV0      = SUM(i, PWM0(i)*M0(i)) - SUM(i, PWE0(i)*E0(i)) ;
  GR0        = SUM(i, TM0(i)*ER0*PWM0(i)*M0(i) ) + 
               SUM(i,TD0(i)*PDD0(i)*D0(i)) - 
               SUM(i, TE0(i)*ER0*PWE0(i)*E0(i)) ;
  Y0         = SUM(i, PX0(i)*X0(i)) +  ER0*FSAV0 + GR0 ;                 

DISPLAY D0, Q0, FSAV0, GR0, Y0 ;

*## INITIALIZE FACTOR MARKETS

* assign output 
* force factor income to equal the sales revenue available to pay factors
   FS0(f)       = SUM(i, FDSC0(i,f)) ;   
   FCTRY0(i,f)  = (FCTRY0(i,f)/SUM(ff, FCTRY0(i,ff)))*PX0(i)*X0(i) ;
   YFCTR0(f)    = SUM(i, FCTRY0(i,f)) ;
   YFSECT0(i)   = SUM(f, FCTRY0(i,f)) ;
   WF0(f)       = YFCTR0(f)/FS0(f) ;
   WFDIST0(i,f) = (FCTRY0(i,f)/FDSC0(i,f))/WF0(f) ;

DISPLAY YFSECT0, FDSC0, FS0, FCTRY0, YFCTR0, WFDIST0, X0 ;

*## ASSIGN AND CALIBRATE EFFICIENCY WAGE PARAMETERS

TABLE NSCPARM(i,f,*) PARAMETERS IN THE NSC
                   D1     D2     QR     UTS  
mfg.lab           .10    .30    .05      .2  
;

D1(i,f)            = NSCPARM(i,f,"D1") ;
D2(i,f)            = NSCPARM(i,f,"D2") ;
QR(i,f)            = NSCPARM(i,f,"QR") ;
UTS(i,f)           = NSCPARM(i,f,"UTS") ;
IR                 = .05 ;

*## CALIBRATE THE NSC PARAMETERS SO THE APPROPRIATE WAGE MARK-UP IS
*## WFDIST, THE VALUE READ IN AND CONSISTENT WITH THE VALUE OF PRODUCTION
* redefine uts to be consistent with the assigned value of wfdist
* note, this assumes the value for wfdist is greater than one

UTS(i,f)$ew(i,f) = (WFDIST0(i,f) - 1 )
                  *( (D2(i,f) - D1(i,f))* (FS0("lab") - FDSC0("mfg","lab")) )
                  *( IR*(FS0("lab") - FDSC0("mfg","lab")) + 
                     (D1(i,f) + QR(i,f))*FS0(f) )**(-1) ;

DISPLAY UTS, WFDIST0 ;

*## INITIALIZE WELFARE PARAMETERS
CLES(i)         = PQ0(i)*Q0(i)/Y0 ;
CLIV0           = PROD(i, PQ0(i)**CLES(i)) ;
UTIL0           = PROD(i, Q0(i)**CLES(i)) ;

DISPLAY CLES, CLIV0, UTIL0 ;

**## INITIALIZE DOMESTIC PRICE INDEX 
 PWTS(i)     = D0(i)/SUM(j, D0(j)) ;
 PINDEX0     = SUM(i, pwts(i)*PDD0(i)) ;

*## CALIBRATE CET and CES TRADE AGGREGATION FUNCTIONS
*## CET Composite of Exports and Domestic Good

PARAMETER DENOM2(i)  denominator in calibration program;

 DENOM2(i)      = PED0(i)*E0(i)**(1+RHOT(i)) + PDD0(i)*D0(i)**(1+RHOT(i)) ;
 ALPHA(i)$E0(i) = (PED0(i)*E0(i)**(1+RHOT(i)) )/DENOM2(i) ;
 ALPHA(i)$(E0(i) EQ 0 ) = 0 ;             
 A2(i)          = X0(i)/( ALPHA(i)*E0(i)$E0(i)**(-RHOT(i))+(1 - ALPHA(i))*D0(i)                                   
                  **(-RHOT(i)) )**(-1/RHOT(i))   ;               
 
DISPLAY DENOM2, ALPHA, A2;
DISPLAY E0, D0 ;
*## Armington CES Composite of Domestic and Imported Variety
* beta is coefficient on imports, the coefficient on domestic
* consumption is 1 - sum of betas

 PARAMETER DENOM1(i)  denominator in beta equation ;

 DENOM1(i)      =  PMD0(i)*M0(i)**(1+RHOC(i))  +
                   PDT0(i)*D0(i)$D0(i)**(1+RHOC(i)) ;

 BETA(i)$M0(i)  = (PMD0(i)*M0(i)**(1+RHOC(i)) ) / DENOM1(i) ;
                          
 BETA(i)$(M0(i) EQ 0 ) = 0 ;

 A1(i)          = Q0(i)/(BETA(i)*M0(i)**(-RHOC(i))  +
                  (1 - BETA(i))*D0(i)**(-RHOC(i)) )**(-1/RHOC(i)) ;
                     

 DISPLAY DENOM1, BETA, A1 ;

*## CES PRODUCTION FUNCTION FOR AN ARBITRARY NUMBER OF ARGUMENTS
*## get values for delta from the producer's cost minimization problem
*## get A3 from the constraint

DELTA(i,f)$FDSC0(i,f) = ( WFDIST0(i,f)*WF0(f)*FDSC0(i,f)**
                        (1 + RHOP(i)) )/SUM(ff, WFDIST0(i,ff)
                        *WF0(ff)*FDSC0(i,ff)**(1+RHOP(i)) ) ;                                                         
                        
DELTA(i,f)$(FDSC0(i,f) EQ 0) = 0.0 ;
                                                                
A3(i)                 = X0(i)/( SUM(f$(FDSC0(i,f)), DELTA(i,f)*FDSC0(i,f)
                        **(-RHOP(i))) )**(-1/RHOP(i));                         
SIGMAP(i)             = 1/(1 + RHOP(i)) ;

display DELTA, A3, SIGMAP ;

*## ASSIGN AND CALIBRATE UNION PARAMETERS
* given the wfdist0, find the value for lbar

 LBAR(i,f)$UN(i,f)  = FDSC0(i,"lab") - 
                      ( (WFDIST0(i,"lab") - 1)/WFDIST0(i,"lab") )*
                      (FDSC0(i,"lab")/MU(i,"lab"))*(1 - MU(i,"lab"))*
                      SIGMAP(i) ;
 WSUB(i,f)          = 0.0 ;

 DISPLAY LBAR ;

*## CALIBRATION CHECK
 PARAMETER X0CHECK(i)  CET Aggregation check
           X0DIFF(i) 
           Q0CHECK(i)  Armington Aggregation check
           Q0DIFF(i) 
           PROCHECK(i) Production function check
           PRODIFF(i)
;

X0CHECK(i) = A2(i)*(ALPHA(i)*E0(i)$E0(i)**(-RHOT(i))+ (1 - ALPHA(i))*D0(i)                
             **(-RHOT(i)))**(-1/RHOT(i))   ;
            
X0DIFF(i)  = X0CHECK(i) - X0(i) ;

Q0CHECK(i) = A1(i)*( BETA(i)*M0(i)**(-RHOC(i)) +(1 - BETA(i))*D0(i)               
             **(-RHOC(i)) )**(-1/RHOC(i))  ;
             
Q0DIFF(i)  = Q0CHECK(i) - Q0(i) ;

* for CES function
PROCHECK(i) = A3(i)* SUM(f, DELTA(i,f)*FDSC0(i,f)**(-RHOP(i)) )
              **(-1/RHOP(i))  ;
                 
              
PRODIFF(i)  = PROCHECK(i) - X0(i) ;

DISPLAY X0DIFF, Q0DIFF, PRODIFF  ;

*## VARIABLE DECLARATION

Positive Variables 
    E(i)         Exports
    DD(i)        Domestic Demand
    M(i)         Imports 
    X(i)         Composite Production
    PX(i)        Producer Price Index
    PQ(i)        Composite Price Index 
    PWE(i)       World Export Price 
    PWM(i)       World Import Price
    PED(i)       Price of Exported Good including subsidy
    PMD(i)       Price of Imported Good including tariff
    PDT(i)       Price of Domestic Good including tax
    PDD(i)       Price of Domestic Good net of tax
    PINDEX       Price Index
    ER           Exchange Rate
    Y            GNP

* Factor Market Variables
    FDSC(i,f)    Factor Demands
    FCTRY(i,f)   Factor Income
    FS(f)        Factor Supply
    WF(f)        Factor Return
    WFDIST(i,f)  Factor Differential  
    MC(i,f)      Wage Mark-up Corresponding to the Monitor
    MONITOR(i,f) Total Monitoring Fees

* Union 
  UNION(i,f)   Union's Utility 
;

 Free Variables
    FSAV         Trade Balance
    Q(i)         Composite Consumption
    TM(i)        Import Tariff
    TE(i)        Export Subsidy
    TD(i)        Domestic Good Tax
    GR           Government Net Revenue
    WALRAS1      Walras law variable 
    WELFARE      Objective function

* Welfare Variables
    CLIV         Cost of Living Index at Current Prices
    UTIL         Utility
    EV           Equivalent Variation
    YN           New Income in EV calc
 ;

*## VARIABLE INITIALIZATION
  DD.L(i)    = D0(i) ;
  M.L(i)     = M0(i) ;
  E.L(i)     = E0(i) ;
  X.L(i)     = X0(i) ;
  Q.L(i)     = Q0(i) ;
  PWM.L(i)   = PWM0(i) ;
  PWE.L(i)   = PWE0(i) ;
  PED.L(i)   = PED0(i) ;
  PMD.L(i)   = PMD0(i) ;
  PDD.L(i)   = PDD0(i) ;
  PINDEX.L   = PINDEX0 ;
  PX.L(i)    = 1 ;
  PQ.L(i)    = PQ0(i) ;
  ER.L       = ER0;
  PDT.L(i)   = PDT0(i) ;
  WALRAS1.L  = 0.0 ;
  FSAV.L     = FSAV0;
  GR.L       = GR0 ;
  Y.L        = Y0 ;

*Initialize Factor Market Variables
    FDSC.L(i,f)   = FDSC0(i,f) ;
    FCTRY.L(i,f)  = FCTRY0(i,f) ;
    FS.L(f)       = FS0(f) ;
    WF.L(f)       = WF0(f) ;
    WFDIST.L(i,f) = WFDIST0(i,f) ;
    MC.L(i,f)       = 1.0 ;
    MONITOR.L(i,f)  = 0.0 ;

*Initialize Union Utility Value 
   UNION.L(i,f)$un(i,f) = (WF.L(f)*WFDIST0(i,f) - WF.L(f) )**MU(i,f)*
                          (FDSC0(i,f) - LBAR(i,f))**(1 - MU(i,f)) ;

* Initialize Welfare Variables
    CLIV.L        = CLIV0 ;
    UTIL.L        = UTIL0 ;
    EV.L          = 0.0 ;
    YN.L          = Y.L ;

*## EQUATION DECLARATION

 Equations
  CET(i)       Composite Production
  CONS(i)      Composite Consumption
  EXPRAT(i)    Ratio of Exports to Domestic Supply
  IMPRAT(i)    Ratio of Imports to Domestic Supply
  EXCH(i)      Composite good balance
  PEXP(i)      Price of Exported Good
  PIMP(i)      Price of Imported Good
  PDOM(i)      Price of Domestic Good
  PDTEQ(i)     Price of Domestic Good Including Tax
  PINDEXEQ     Price Index 
  GREQ         Government Revenue Constraint
  YEQ          Definition of GNP
  BOP          Balance of trade constraint
  REV          Revenue and expenditure constraint
  OBJECT       Objective function

* Product Market Equations
   DEMAND(i)    Demand for composite good

* Factor Market Equations
  PRO(i)       Production Function
  PMAX(i,f)    First Order Conditions for Profit Max
  YFDEF(i,f)   Factor Income
  FMEQUIL(f)   Factor Market Equilibrium

* Efficiency Wage Equations
  NSC(i,f)     No Shirking Condition

* Union Equations
  UTILITY(i,f) Utility Over Wages and Hired
  WMU(i,f)     Wage Mark-up From FOC for Utility Max
 
* Welfare Equations
  CLIVEQ       Cost of Living Index
  UTILEQ       Utility 
  EVEQ         Equivalent Variation 
  YNEQ         New income in EV
  ;


  CET(i)..     X(i) =E= A2(i)*( ALPHA(i)*E(i)**(-RHOT(i)) + (1 - ALPHA(i))*DD(i)
               **(-RHOT(i)) )**(-1/RHOT(i)) ;

  EXPRAT(i)$E0(i)..  E(i) =E= DD(i)*( PDD(i)/PED(i)*ALPHA(i)/(1 - ALPHA(i)) )
                     **(1/(1+RHOT(i))) ;
 
  CONS(i)..    Q(i) =E= A1(i)*(BETA(i)*M(i)**(-RHOC(i)) +
               (1 - BETA(i))*DD(i)**(-RHOC(i)) )**(-1/RHOC(i)) ;                 
                  
  IMPRAT(i)$M0(i)..  M(i) =E= DD(i)*( PDT(i)/PMD(i)*BETA(i)/(1 - BETA(i)) )
                    **(1/(1+RHOC(i))) ;                                                         
                            
  PEXP(i)..    PED(i) =E= ER*PWE(i)*(1+TE(i));

  EXCH(i)..    PX(i)*X(i) =E= PED(i)*E(i) + PDD(i)*DD(i) ;                 
 
  PIMP(i)..    PMD(i) =E= PWM(i)*ER*(1+TM(i)) ;

  PDOM(i)..    PQ(i)*Q(i) =E= PDT(i)*DD(i) + PMD(i)*M(i) ;
               
  PDTEQ(i)..   PDT(i) =E= (1+TD(i))*PDD(i);

  PINDEXEQ..   PINDEX =E= SUM(i, PWTS(i)*PDD(i)) ;

  GREQ..       GR =E= SUM(i, TM(i)*ER*PWM(i)*M(i)) + 
               SUM(i, TD(i)*PDD(i)*DD(i)) - SUM(i, TE(i)*ER*PWE(i)*E(i)) -
               SUM((i,f), WF(f)*WFDIST(i,f)*FDSC(i,f)*WSUB(i,f)) ;
               
  YEQ..        Y =E= SUM((i,f), FCTRY(i,f) + MONITOR(i,f))+ ER*FSAV + GR;

  BOP..        FSAV =E= SUM(i, PWM(i)*M(i)) - SUM(i, PWE(i)*E(i)) ;
               
  REV..        Y  =E= SUM(i, PQ(i)*Q(i)) + WALRAS1 ;

  OBJECT..      WELFARE =E=  WALRAS1**2 ;

* Product Market Equations

 DEMAND(i).. PQ(i)*Q(i)  =E= CLES(i)*Y ;

* Factor Market Equations
* CES Function

  PRO(i)..     X(i) =E= A3(i)*( SUM(f, DELTA(i,f)*FDSC(i,f)
                 **(-RHOP(i))) )**(-1/RHOP(i))  ;

  PMAX(i,f)$WFDIST0(i,f)..  
               WF(f)*WFDIST(i,f)*(1 - WSUB(i,f)) =E= 
               (PX(i) - SUM(ff, MONITOR(i,ff)/X(i)) )
               *A3(i)*
               (SUM(ff$FDSC0(i,ff), DELTA(i,ff)*FDSC(i,ff)**(-RHOP(i)))  )
               **((-1/RHOP(i)) - 1)*DELTA(i,f)*FDSC(i,f)**(-RHOP(i)-1);                                    
$ontext
* Cobb-Douglas Function
 PRO(i)..   X(i)   =E= A3(i)*PROD(f$DELTA(i,f), FDSC(i,f)**DELTA(i,f));
              
 PMAX(i,f)$WFDIST0(i,f)..  WF(f)*WFDIST(i,f)*FDSC(i,f) =E=
                           X(i)*PX(i)*DELTA(i,f) ;
$offtext
 YFDEF(i,f)..   FCTRY(i,f) =E=  WF(f)*WFDIST(i,f)*FDSC(i,f);               

 FMEQUIL(f)..   SUM(i, FDSC(i,f)) =E= FS(f) ;

*Efficiency Wage Equations

 NSC(i,f)$ew(i,f)..  WFDIST(i,f) =E=  1 + UTS(i,f)*IR/
                     ( (D2(i,f) - D1(i,f)) ) + 
                     UTS(i,f)*(D1(i,f) + QR(i,f))*FS(f)/
                     ( (D2(i,f) - D1(i,f) )*
                     (FS("lab") -  FDSC("mfg","lab")) ) ;
*Union Equations
 UTILITY(i,f)$UN(i,f)..  UNION(i,f) =E=  (WFDIST(i,f)*WF(f) - WF(f))**MU(i,f)*
                         (FDSC(i,f) - LBAR(i,f))**(1 - MU(i,f)) ;

 WMU(i,f)$UN(i,f)..      (WFDIST(i,f) - 1 ) =E=  WFDIST(i,f)*
                         MU(i,f)/( (1 - MU(i,f))*SIGMAP(i))*
                         (FDSC(i,f) - LBAR(i,f))/FDSC(i,f) ;

*Welfare Equations
  CLIVEQ..     CLIV  =E= PROD(i, PQ(i)**CLES(i)) ;

  UTILEQ..     UTIL  =E= PROD(i, Q(i)**CLES(i)) ;
  
  EVEQ..       EV    =E= Y0 - YN ;
 
  YNEQ..       UTIL  =E=  PROD(i, (CLES(i)*YN/PQ0(i))**CLES(i) ) ;

*## VARIABLE BOUNDS
  PDD.LO(i)             = 0.0 ;
  PDT.LO(i)             = 0.0 ;
  PED.LO(i)             = 0.0 ;
  PMD.LO(i)             = 0.0 ;
  UTIL.LO               = 0.0 ;
  CLIV.LO               = 0.0 ;
                  
                 
*## CLOSURE CONDITIONS
 
* Maintain Initial Zeros
  M.FX(i)$(M0(i) EQ 0)             = 0.0 ;
  E.FX(i)$(E0(i) EQ 0)             = 0.0 ;
  FDSC.FX(i,f)$(WFDIST0(i,f) EQ 0) = 0.0 ;

* numeraire
  PINDEX.FX         = 1.0 ;

* Trade Closure
  FSAV.FX           = FSAV0 ;
* ER.FX             = ER.L ;

* Small Country Case
  PWE.FX(i)         = PWE0(i) ;
  PWM.FX(i)         = PWM0(i) ;

* Tax Rates
  TM.FX(i)          = TM0(i) ;
  TE.FX(i)          = TE0(i) ;
  TD.FX(i)          = TD0(i) ;

* Factor Markets
  FS.FX(f)                   = FS0(f) ;
  MONITOR.FX(i,f)            = MONITOR.L(i,f) ;
  MC.FX(i,f)                 = MC.L(i,f) ;
  WFDIST.FX(i,f)$nun(i,f)    = WFDIST0(i,f) ;
  UNION.FX(i,f)$nun(i,f)     = 0.0 ;

 Options iterlim=10000,limrow=5,limcol=0;
   option nlp=conopt; 

 Model Trade /ALL/;
*trade.optfile   = 1 ;
*trade.holdfixed = 1 ;

* Solve Trade Using NLP MAXIMIZING WELFARE ;
  Solve Trade Using NLP minimizing WELFARE ;

DISPLAY WALRAS1.L ;

SET 
   rq quantity table   /x,q,dd,e,m/
   rp price table      /px,pq,pdt,ped,pwe,pmd,pwm/
   rf factor table     /fd_L,fd_K,wf_L,wf_K,wfdist_L,wfdist_K/
   ra aggregate table  /er,util,ev/

PARAMETER 
          QUANT0(rq,i)    BASE OUTPUT QUANTITIES
          PRICE0(rp,i)    BASE OUTPUT PRICES
          FACTOR0(rf,i)   BASE FACTOR MARKETS 
          AGG0(ra)        BASE AGGREGATES
;
QUANT0("x",i)   = X.L(i) ;
QUANT0("q",i)   = Q.L(i) ;
QUANT0("dd",i)  = DD.L(i) ;
QUANT0("e",i)   = E.L(i) ;
QUANT0("m",i)   = M.L(i) ;

PRICE0("px",i)  = PX.L(i) ;
PRICE0("pq",i)  = PQ.L(i) ;
PRICE0("pdt",i) = PDT.L(i) ;
PRICE0("ped",i) = PED.L(i) ;
PRICE0("pwe",i) = PWE.L(i) ;
PRICE0("pmd",i) = PMD.L(i) ;
PRICE0("pwm",i) = PWM.L(i) ;

FACTOR0("fd_L",i)      = FDSC.L(i,"lab") ;
FACTOR0("fd_K",i)      = FDSC.L(i,"cap") ;
FACTOR0("wf_L",i)      = SUM(j, WF.L("lab")*WFDIST.L(j,"lab")*FDSC.L(j,"lab"))/
                         SUM(j, FDSC.L(j,"lab"))  ;
FACTOR0("wf_K",i)      = SUM(j, WF.L("cap")*WFDIST.L(j,"cap")*FDSC.L(j,"cap"))/
                         SUM(j, FDSC.L(j,"cap"))  ;
FACTOR0("wfdist_L",i)  = WFDIST.L(i,"lab") ;
FACTOR0("wfdist_K",i)  = WFDIST.L(i,"cap") ;

AGG0("er")   = ER.L ;
AGG0("util") = UTIL.L ;
AGG0("ev")   = EV.L ;

DISPLAY QUANT0, PRICE0, FACTOR0, AGG0 ;

